!= Matthews Υǡ˴Ťɽ̥٥
!
!= set surface albedo based on data by Matthews
!
! Authors::   Yoshiyuki O. Takahashi
! Version::   $Id: albedo_Matthews.f90,v 1.2 2010-02-24 08:07:54 yot Exp $
! Tag Name::  $Name: dcpam5-20100424 $
! Copyright:: Copyright (C) GFD Dennou Club, 2008-2009. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]
!

module albedo_Matthews

  ! ⥸塼 ; USE statements
  !

  ! ̷ѥ᥿
  ! Kind type parameter
  !
  use dc_types, only: DP, &      ! ټ¿. Double precision.
    &                 STRING     ! ʸ.       Strings.


  ! ʻ
  ! Grid points settings

  ! ʸ ; Declaration statements
  !
  implicit none
  private

  ! ³
  ! Public procedure
  !
  public:: SetAlbedoMathews

  ! ѿ
  ! Public variables
  !
  logical, save, public:: albedo_mathews_inited = .false.
                              ! ե饰.
                              ! Initialization flag

  ! ѿ
  ! Private variables
  !
  logical   , save      :: flag_annual_mean

  real(DP)  , save      :: OceanAlbedo

  integer , parameter :: NAlbType = 32
  integer , parameter :: NSeason = 4
  real(DP), save      :: a_Data_DOY( NSeason )
  real(DP), save      :: xy_Data_Albedo( NSeason, 0:NAlbType )

  data a_Data_DOY / 0.0d0, 90.0d0, 181.0d0, 273.0d0 /

  !
  ! Matthews, 1985, NASA Technical memorandum #86199
  ! http://ntrs.nasa.gov/search.jsp?R=60233&id=1&qs=Ntt%3DMatthews%26Ntk%3Dall%26Ntx%3Dmode%2520matchall%26N%3D31%2B235%26Ns%3DHarvestDate%257c1
  !
  !    win.  spr.  sum.  fall
  !    Jan.  Apr.  Jul.  Oct.   for nothern hemisphere
  !    Jul.  Oct.  Jan.  Apr.   for southern hemisphere
  data xy_Data_Albedo / &
    10,   10,   10,   10, & !  0  The values for 0 are not included in Matthes compilation.
    11,   11,   11,   11, & !  1
    11,   11,   11,   11, & !  2
    11,   11,   11,   11, & !  3
    12,   12,   12,   12, & !  4
    12,   13,   14,   13, & !  5
    17,   14,   13,   14, & !  6
    13,   14,   16,   13, & !  7
    11,   12,   15,   12, & !  8
    18,   16,   15,   16, & !  9
    12,   15,   18,   13, & ! 10
    12,   15,   18,   13, & ! 11
    28,   32,   28,   28, & ! 12
    15,   13,   12,   13, & ! 13
    14,   14,   16,   14, & ! 14
    20,   18,   17,   18, & ! 15
    14,   14,   17,   14, & ! 16
    15,   15,   18,   15, & ! 17
    15,   15,   18,   15, & ! 18
    17,   20,   20,   17, & ! 19
    17,   20,   20,   17, & ! 20
    28,   32,   28,   28, & ! 21
    12,   12,   17,   15, & ! 22
    14,   15,   17,   15, & ! 23
    14,   15,   16,   14, & ! 24
    16,   18,   25,   20, & ! 25
    17,   17,   20,   17, & ! 26
    16,   20,   20,   18, & ! 27
    16,   20,   20,   18, & ! 28
    16,   20,   20,   18, & ! 29
    30,   30,   30,   30, & ! 30
    75,   75,   75,   75, & ! 31
    16,   18,   20,   18  & ! 32
    /


  character(*), parameter:: module_name = 'albedo_Matthews'
                              ! ⥸塼̾.
                              ! Module name
  character(*), parameter:: version = &
    & '$Name: dcpam5-20100424 $' // &
    & '$Id: albedo_Matthews.f90,v 1.2 2010-02-24 08:07:54 yot Exp $'
                              ! ⥸塼ΥС
                              ! Module version

  ! INTERFACE ʸ ; INTERFACE statements
  !

contains

  subroutine SetAlbedoMathews( xy_SurfCond, xy_SurfAlbedo )

    ! ⥸塼 ; USE statements
    !

    ! ʻ
    ! Grid points settings
    !
    use gridset, only: imax, & ! ٳʻ.
                               ! Number of grid points in longitude
      &                jmax, & ! ٳʻ.
                               ! Number of grid points in latitude
      &                kmax    ! ľؿ.
                               ! Number of vertical level

    ! դӻμ갷
    ! Date and time handler
    !
    use dc_date, only: DC_DIFFTIME, EvalDay, EvalSec

    use dc_date_types, only: DAY_SECONDS, YEAR_DAYS

    ! 
    ! Time control
    !
    use timeset, only: TimeN

    ! ʸ ; Declaration statements
    !
    integer , intent(in ) :: xy_SurfCond  ( 0:imax-1, 1:jmax )
                              ! ɽ (0: , 1: ).
                              ! Surface condition (0: fixed, 1: variable)
    real(DP), intent(out) :: xy_SurfAlbedo( 0:imax-1, 1:jmax )
                              ! ɽ٥.
                              ! Surface albedo

    ! ѿ
    ! Work variables
    !
    real(DP):: SecOfYear
    real(DP):: a_Data_SOY_Ex( 0:nseason+1 )
    real(DP):: xya_SurfAlbedoLocal( 0:imax-1, 1:jmax, 1:2 )
    integer :: i               ! ˲ DO 롼Ѻѿ
                              ! Work variables for DO loop in longitude
    integer :: j               ! ˲ DO 롼Ѻѿ
                              ! Work variables for DO loop in latitude
    integer :: l
    integer :: t
    integer :: tindex
    integer :: a_tindex(1:2)


    if ( .not. albedo_mathews_inited ) call SetAlbedoInit


    if ( flag_annual_mean ) then

      !
      ! Now, annual mean value is used, temporarily.
      !
      xy_SurfAlbedo = 0.0d0

      do l = 1, nseason
        do j = 1, jmax
          do i = 0, imax-1
            xy_SurfAlbedo(i,j) = xy_SurfAlbedo(i,j) + xy_Data_Albedo( l, xy_SurfCond(i,j) )
          end do
        end do
      end do

      xy_SurfAlbedo = xy_SurfAlbedo / dble( nseason )

    else

      SecOfYear = mod( EvalSec( TimeN ), YEAR_DAYS * DAY_SECONDS )
      if ( SecOfYear > YEAR_DAYS * DAY_SECONDS ) SecOfYear = YEAR_DAYS * DAY_SECONDS

      a_Data_SOY_Ex(0) = ( 0.0d0 - ( YEAR_DAYS - a_Data_DOY(nseason) ) ) * DAY_SECONDS
      do t = 1, nseason
        a_Data_SOY_Ex(t) = a_Data_DOY(t) * DAY_SECONDS
      end do
      a_Data_SOY_Ex(nseason+1) = ( YEAR_DAYS + a_Data_DOY(1) ) * DAY_SECONDS


      a_tindex(1) = 0
      a_tindex(2) = 1
      do t = 1, nseason
        if ( a_Data_DOY(t) * DAY_SECONDS <= SecOfYear ) then
          a_tindex(1) = t
          a_tindex(2) = t+1
        end if
      end do

      do t = 1, 2
        ! for northern hemisphere
        tindex = a_tindex(t)
        if ( tindex == 0 ) then
          tindex = nseason
        else if ( tindex == nseason+1 ) then
          tindex = 1
        else
          tindex = tindex
        end if
        do j = jmax/2+1, jmax
          do i = 0, imax-1
            xya_SurfAlbedoLocal(i,j,t) = xy_Data_Albedo( tindex, xy_SurfCond(i,j) )
          end do
        end do
        ! for southern hemisphere
        tindex = a_tindex(t) + nseason / 2
        if ( tindex > nseason ) tindex = tindex - nseason
        if ( tindex == 0 ) then
          tindex = nseason
        else if ( tindex == nseason+1 ) then
          tindex = 1
        else
          tindex = tindex
        end if
        do j = 1, jmax/2
          do i = 0, imax-1
            xya_SurfAlbedoLocal(i,j,t) = xy_Data_Albedo( tindex, xy_SurfCond(i,j) )
          end do
        end do
      end do

      xy_SurfAlbedo =                                                   &
        &   ( xya_SurfAlbedoLocal(:,:,2) - xya_SurfAlbedoLocal(:,:,1) ) &
        & / ( a_Data_SOY_Ex(a_tindex(2)) - a_Data_SOY_Ex(a_tindex(1)) ) &
        & * ( SecOfYear                  - a_Data_SOY_Ex(a_tindex(1)) ) &
        & + xya_SurfAlbedoLocal(:,:,1)

    end if


  end subroutine SetAlbedoMathews

  !**************************************************************************************

  subroutine SetAlbedoInit

    ! NAMELIST եϤ˴ؤ桼ƥƥ
    ! Utilities for NAMELIST file input
    !
    use namelist_util, only: namelist_filename, NmlutilMsg, NmlutilAryValid

    ! ե
    ! File I/O support
    !
    use dc_iounit, only: FileOpen

    ! å
    ! Message output
    !
    use dc_message, only: MessageNotify

    ! ѿ
    ! Work variables
    !
    integer:: unit_nml        ! NAMELIST ե륪ץֹ.
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST ɤ߹߻ IOSTAT.
                              ! IOSTAT of NAMELIST read

    ! NAMELIST ѿ
    ! NAMELIST group name
    !
    namelist /albedo_Matthews_nml/ &
      & flag_annual_mean, &
      & OceanAlbedo
          !
          ! ǥեͤˤĤƤϽ³ "surface_flux_bulk#SurfFluxInit"
          ! Υɤ򻲾ȤΤ.
          !
          ! Refer to source codes in the initialization procedure
          ! "surface_flux_bulk#SurfFluxInit" for the default values.
          !


    ! ǥեͤ
    ! Default values settings
    !
    flag_annual_mean = .false.
    OceanAlbedo      = 0.1d0


    ! NAMELIST ɤ߹
    ! NAMELIST is input
    !
    if ( trim(namelist_filename) /= '' ) then
      call FileOpen( unit_nml, &          ! (out)
        & namelist_filename, mode = 'r' ) ! (in)

      rewind( unit_nml )
      read( unit_nml, &                ! (in)
        & nml = albedo_Matthews_nml, &  ! (out)
        & iostat = iostat_nml )        ! (out)
      close( unit_nml )

      call NmlutilMsg( iostat_nml, module_name ) ! (in)
    end if



    xy_Data_Albedo = xy_Data_Albedo * 1.0d-2


    xy_Data_Albedo(:,0) = OceanAlbedo



    !  ; Print
    !
    call MessageNotify( 'M', module_name, '----- Initialization Messages -----' )
    call MessageNotify( 'M', module_name, '  flag_annual_mean = %b', l = (/ flag_annual_mean /) )
    call MessageNotify( 'M', module_name, '  OceanAlbedo      = %f', d = (/ OceanAlbedo      /) )
    call MessageNotify( 'M', module_name, '-- version = %c', c1 = trim(version) )


    albedo_mathews_inited = .true.

  end subroutine SetAlbedoInit

  !**************************************************************************************

end module albedo_Matthews
